#!/usr/bin/env python3
#
# Copyright (C) 2023 Savoir-faire Linux Inc.
#
# Author: Xavier Jouslin de Noray <xavier.jouslindenoray@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Creates packaging targets for a distribution and architecture.
# This helps reduce the length of the top Makefile.
#
from os import makedirs, path
import argparse
from typing import Optional, Union, Tuple, Dict

import utils
from pluginCertificate import PluginCertificate
from certificate import Certificate

def parse() ->argparse.Namespace:
    parser = argparse.ArgumentParser(description='Create a certificate.')
    sub_parser = parser.add_subparsers(dest='subcommand')
    parser.add_argument('save', help='Path to save file', type=str)
    parser.add_argument('--plugin',
                            action='store_true',
                            help='Specified if it\'s a plugin certificate.',
                        )
    create_arg_parser = sub_parser.add_parser("create")
    create_arg_group = create_arg_parser.add_argument_group('create')
    create_arg_group.add_argument('--subject',
                                  help='Specified subject of a new certificate.',
                                  required=True,
                                  type=str
                                )
    create_arg_group.add_argument('--issuer',
                                  help='Specified issuer path to certificate and private key.',
                                  type=str
                                )
    sign_arg_parser = sub_parser.add_parser("sign")
    sign_arg_group = sign_arg_parser.add_argument_group('sign')
    sign_arg_group.add_argument('--path',
                                help='Specified the path to the file to sign',
                                required=True,
                                type=str
                            )
    sign_arg_group.add_argument('--issuer',
                                help='Specified issuer path to certificate and private key.',
                                type=str
                            )
    return parser.parse_args()


def create(subject: str, issuer: Optional[str], is_plugin: bool) -> Optional['Certificate']:
    cert = None
    if utils.isEmpty(subject or (not is_plugin and utils.isEmpty(issuer))):
        raise ValueError
    if utils.isEmpty(issuer):
        cert = Certificate.create(subject)
    elif issuer is not None:
        ca = Certificate.load(issuer)
        cert = Certificate.create(subject, ca)
    return cert

def sign(path_to_file: str, issuer: str, plugin: bool) -> Tuple[bytes, bytes]:
    cert = Certificate.load(issuer)
    if plugin:
        cert = PluginCertificate.load(cert, path_to_file)
        signature = cert.sign()
    else:
        signature = cert.sign(path_to_file)
    return signature

def save(cert: Optional[Union['Certificate', 'PluginCertificate']], signature: Optional[Union[Tuple[bytes, bytes], Dict[str, Dict[str, bytes]]]], save_file: Optional[str], is_plugin: bool) -> None:
    if save_file is None:
        raise ValueError
    path_to_file = path.abspath(path.dirname(save_file))
    if not path.exists(path_to_file):
        makedirs(path_to_file)
    if not utils.isEmpty(cert) or (is_plugin and not utils.isEmpty(signature)):
        if path.isdir(save_file):
            save_file += 'plugin.jpl' if is_plugin else 'certificate'
        if not is_plugin:
            cert.save(save_file)
        else:
            cert.save(save_file, signature)
    if not utils.isEmpty(signature) and not is_plugin:
        if path.isdir(save_file):
            save_file = path.join(path_to_file, 'signature')
        utils.save(f'{save_file}.sig', signature[0], 'wb')

def main():
    cert = None
    signature = None
    parsed_args = parse()
    if parsed_args.subcommand == 'create':
        cert = create(parsed_args.subject, parsed_args.issuer, parsed_args.plugin)
    elif parsed_args.subcommand == 'sign':
        signature = sign(parsed_args.path, parsed_args.issuer, parsed_args.plugin)
    if not utils.isEmpty(parsed_args.save) and (not signature is None or not cert is None):
        if parsed_args.plugin and parsed_args.subcommand == 'sign':
            cert = Certificate.load(parsed_args.issuer)
            cert = PluginCertificate.load(cert, parsed_args.path)
        save(cert,
             signature,
             parsed_args.save,
             parsed_args.subcommand == 'sign' and parsed_args.plugin
            )

if __name__ == '__main__':
    main()
