#
# Copyright (C) 2023 Savoir-faire Linux Inc.
#
# Author: Xavier Jouslin de Noray <xavier.joulindenoray@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Creates packaging targets for a distribution and architecture.
# This helps reduce the length of the top Makefile.
#
from OpenSSL import crypto
from typing import Optional, Tuple

from utils import save, read


class Certificate:
    def __init__(self, key: crypto.PKey, cert: crypto.X509) -> None:
        self._cert = cert
        self._key = key

    @property
    def cert(self) -> Tuple[crypto.PKey, crypto.X509]:
        return self._key, self._cert

    @staticmethod
    def load(file_path: str) -> 'Certificate':
        cert = crypto.load_certificate(crypto.FILETYPE_PEM, read(f'{file_path}.crt', 'rb'))
        key = crypto.load_privatekey(crypto.FILETYPE_PEM, read(f'{file_path}.key', 'rb'))
        return Certificate(key, cert)

    @staticmethod
    def create(subject: str, issuer: Optional['Certificate'] = None) -> 'Certificate':
        key = crypto.PKey()
        key.generate_key(crypto.TYPE_RSA, 4096)
        cert = crypto.X509()
        subj = cert.get_subject()
        subj.commonName = subject
        issuer_crt = issuer.cert[1] if issuer is not None else cert
        issuer_key = issuer.cert[0] if issuer is not None else key
        if issuer_crt is None:
            raise ValueError
        cert.set_issuer(issuer_crt.get_subject())
        cert.set_pubkey(key)
        cert.gmtime_adj_notBefore(0)
        cert.gmtime_adj_notAfter(365*24*60*60)
        cert.sign(issuer_key, 'sha512')
        return Certificate(key, cert)

    def save(self, path_2_file: str) -> None:
        if self._cert is None:
            raise ValueError
        save(f'{path_2_file}.crt', crypto.dump_certificate(crypto.FILETYPE_PEM, self._cert), 'wb')
        save(f'{path_2_file}.key', crypto.dump_privatekey(crypto.FILETYPE_PEM, self._key), 'wb')

    def sign(self, file: str) -> Tuple[bytes, bytes]:
        if self._cert is None or self._key is None:
            raise ValueError
        priv_key = self._key
        with open(file, "rb") as f:
            data = f.read()
        signature = crypto.sign(priv_key, data, "sha512")
        if self.verify_signature(signature, data):
            return signature , data
        else:
            raise ValueError

    def verify_certificate(self, subject: crypto.X509) -> bool:
        if self._cert is None:
            return False
        try:
            verified = subject.to_cryptography().verify_directly_issued_by(
                self._cert.to_cryptography()
                )
        except:
            return False
        return verified is None

    def verify_signature(self, signature: bytes, data: bytes) -> bool:
        try:
            return crypto.verify(self._cert, signature, data, "sha512") is None
        except:
            return False
